package com.plantas.plantasalicante;

import java.util.Vector;

import android.app.Activity;
import android.database.Cursor;
import android.net.Uri;

/**
 * Clase que permite gestionar las plantas y el acceso al ContentProvider
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class AlmacenPlantas {

	private Activity activity;

	/** 
	 * Constructor de AlmacenPlantas
	 * @param activity Actividad que ha solicitado el almacen de plantas
	 */
	public AlmacenPlantas(Activity activity){
		this.activity = activity;
	}

	/** 
	 * Metodo que permite solicitar todas las plantas del Content Provider pero solo devuelve el nombre comun, el cientifico y el tipo
	 */
	public Vector<String> listaPlantas() {
		//aqui debemos recuperar el contenido del Content Provider
		Vector<String> listaplantas = new Vector<String>();
		Uri uri = Uri.parse("content://com.plantas.plantasalicante/plantas");
		Cursor cursor = activity.getContentResolver().query(uri,null,null,null,"nombre_comun ASC");

		if(cursor != null){
			while(cursor.moveToNext()){
				String nombre = cursor.getString(cursor.getColumnIndex("nombre_comun"));
				String cientifico = cursor.getString(cursor.getColumnIndex("nombre_cientifico"));
				String tipo = cursor.getString(cursor.getColumnIndex("tipo"));
				listaplantas.add(nombre + ";" + cientifico + ";" + tipo);
			}
		}
		return listaplantas;
	}

	/** 
	 * Metodo que permite solicitar todas las plantas del Content Provider pero solo devuelve el nombre comun, el cientifico y el tipo y los lugares
	 */
	//este metodo devuelve ademas del nombre comun, el indice de lugares
	public Vector<String> listaPlantasZonas() {
		// TODO Auto-generated method stub
		Vector<String> listaplantas = new Vector<String>();
		Uri uri = Uri.parse("content://com.plantas.plantasalicante/plantas");
		Cursor cursor = activity.getContentResolver().query(uri,null,null,null,"nombre_comun ASC");
		if(cursor != null){
			while(cursor.moveToNext()){
				String nombre = cursor.getString(cursor.getColumnIndex("nombre_comun")) + "****" + cursor.getString(cursor.getColumnIndex("nombre_cientifico")) ;
				String lugares = cursor.getString(cursor.getColumnIndex("lugares"));
				String tipo = cursor.getString(cursor.getColumnIndex("tipo"));
				listaplantas.add(nombre + ";" + lugares + ";" + tipo );
			}
		}
		return listaplantas;
	}

	/** 
	 * Metodo que permite recuperar todos los datos de una planta especifica
	 *@param nombrebuscado Planta que se desee buscar
	 */
	public Cursor PlantaBuscada(String nombrebuscado) {
		// TODO Auto-generated method stub
		Uri uri = Uri.parse("content://com.plantas.plantasalicante/plantas/" + nombrebuscado);
		Cursor cursor = activity.getContentResolver().query(uri,null,null,null,null);
		return cursor;
	}

}
