package com.plantas.plantasalicante;

import java.net.URL;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import android.app.Activity;
import android.app.ActivityManager;
import android.app.ActivityManager.RunningServiceInfo;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationManager;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.SoundPool;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;

/**
 * Clase que permite representar la pantalla principal de la aplicacion
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class MainActivity extends Activity {
	private LocationManager manejador;
	private Location mejorLocaliz;
	private static int DOS_MINUTOS = 2*60*1000;
	private MediaPlayer musica;
	private int sonidoboton;
	private SoundPool sP;
	public static AlmacenPlantas almacen;
	private Context context;
	private ProgressDialog progreso;


	/** 
	 * Crea la actividad principal de la aplicacion
	 * @param savedInstanceState Estado anterior de la actividad.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		almacen = new AlmacenPlantas(this);
		context = this;
		//hay que coger las coordenadas GPS y ver si estamos dentro de Alicante
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esGPS = pref.getBoolean("permitirgps", true);
		manejador = (LocationManager) getSystemService(LOCATION_SERVICE);
		//Si el usuario nos lo ha autorizado y hay conexion a Internet obtenemos la ultimos coordenadas GPS
		if(esGPS && verificaConexion(this)){
			String opcion = pref.getString("tipos", "2");
			if(manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) && (opcion.equals("0") || opcion.equals("2"))){

				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.GPS_PROVIDER));
			}
			if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && (opcion.equals("1") || opcion.equals("2"))){
				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
			}
			//ahora comprobamos
			//la consulta debe ir al servicio de google como http://maps.googleapis.com/maps/api/geocode/json?latlng=40.714224,-73.961452&sensor=true
			//este proceso se har con un Asyntask para no bloquear el hilo principal

			TareaGPS tg = new TareaGPS();
			tg.execute(1);
			cargaViewYBotones();
			//FIN IF esGPS
		}
		else{
			Log.e("Plantas","GPS Desactivado");
			cargaViewYBotones();
			Intent e = new Intent(this,GPS_Desactivado.class);
			startActivity(e);
		}
		//lanzamos los servicios si no estan operativos
		if(!isMyServiceRunning(ServicioActualizacion.class)){
			//no esta operativo, lo lanzamos
			Log.e("Plantas","El servicio no estaba activo, arrancandolo");
			startService(new Intent(this,ServicioActualizacion.class));
		}

		if(!isMyServiceRunning(ServicioZonaPlantas.class)){
			//no esta operativo, lo lanzamos
			Log.e("Plantas","El servicio no estaba activo, arrancandolo");
			startService(new Intent(this,ServicioZonaPlantas.class));
		}

	}

	/** 
	 * Permite cargar los botones de la aplicacion y el contenido extra de la ventana principal
	 */
	private void cargaViewYBotones(){
		setContentView(R.layout.activity_main);
		//hay que cambiar los botones con el degradado
		((Button)findViewById(R.id.button5)).setBackgroundResource(R.drawable.degradadoboton);
		((Button)findViewById(R.id.button4)).setBackgroundResource(R.drawable.degradadoboton);
		((Button)findViewById(R.id.button3)).setBackgroundResource(R.drawable.degradadoboton);
		((Button)findViewById(R.id.button2)).setBackgroundResource(R.drawable.degradadoboton);
		((Button)findViewById(R.id.button1)).setBackgroundResource(R.drawable.degradadoboton);
		//preparamos el sonido de los botones al ser pulsados
		sonidoboton = -1;
		sP = new SoundPool(20,AudioManager.STREAM_MUSIC,0);
		sonidoboton = sP.load(this, R.raw.botonclick, 1);
	}

	/**
	 * Clase que se ejecutar en otro hilo y permitir recuperar la ubicacion actual
	 * @version 1.0, 05/11/14
	 * @author Jose Manuel Castellano Dominguez
	 */
	private class TareaGPS extends AsyncTask<Integer,Void,String>{

		/** 
		 * Metodo que se ejecutar antes de empiece la tarea en background
		 */
		protected void onPreExecute(){
			//debemos indicarle que dentro de la actividad actual queremos poner un cuadro de progreso
			//para que el usuario espere
			progreso = new ProgressDialog(MainActivity.this);
			progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
			progreso.setMessage("Localizando sitio.... Por favor espere");
			//el cuadro deberia irse solo tras un maximo de 8 segundos por tanto no puede ser cancelado por el usuario
			progreso.setCancelable(false);
			progreso.show();
		}

		/** 
		 * Metodo que ejecuta la tarea
		 * @param cantidad Estado anterior de la actividad.
		 */
		protected String doInBackground(Integer... cantidad){
			String result = null;
			//hay que comprobar si hay conexion a Internet, si no la hubiera tenemos que cancelar
			Boolean hayconexion=verificaConexion(context);
			if(!hayconexion || mejorLocaliz == null){
				//no podemos conectarnos, abortamos
				progreso.dismiss();
				return null;
			}
			try {
				//probamos a conectarnos al servicio y el XML recibido sacaremos los datos
				URL url= new URL("https://maps.googleapis.com/maps/api/geocode/xml?latlng=" + mejorLocaliz.getLatitude() + "," + mejorLocaliz.getLongitude() +"&sensor=true");	
				SAXParserFactory fabrica = SAXParserFactory.newInstance();
				SAXParser parser = fabrica.newSAXParser();
				XMLReader lector = parser.getXMLReader();
				ManejadorXML manejadorXML = new ManejadorXML();
				lector.setContentHandler(manejadorXML);
				InputSource datos = new InputSource(url.openStream());
				lector.parse(datos);
				//ahora deberemos llamar a una actividad suponiendo que no se cumpla
				String lugar = manejadorXML.getTotalResults();
				progreso.dismiss();
				if(lugar != null){
					//ok ahora distinguimos el lugar
					if(lugar.equals("Otro")){
						//hay que crear un nuevo intent diciendo que la aplicacion es solo para la 
						//provincia de Alicante
						Intent e = new Intent(MainActivity.this,NoAlicante.class);
						startActivity(e);
					}
					else{
						Log.e("Plantas","Estamos en alicante todo OK");	
					}
				}
				else{
					Log.e("Plantas","No encontrado el sitio, se ignora");
					cargaViewYBotones();
				}
				return manejadorXML.getTotalResults();
			}
			catch(Exception e){
				Log.e("Plantas", "Excepcion:" + e.getMessage(), e);
				progreso.dismiss();
			}
			progreso.dismiss();
			return result;
		}

	}

	/**
	 * Clase que permite manejar los datos XML del servicio de geolocalizacion de google
	 * @version 1.0, 05/11/14
	 * @author Jose Manuel Castellano Dominguez
	 */
	public class ManejadorXML extends DefaultHandler{
		private String totalResults;
		private StringBuilder cadena = new StringBuilder();
		private String provincia;

		/** 
		 * Devuelve la cadena encontrada
		 */
		public String getTotalResults(){
			return totalResults;
		}

		/** 
		 * Metodo que empieza a parsear un nuevo elemento
		 * @param uri Cadena de la direccion que empieza
		 * @param nombreLocal El dato que viene dentro de la clave XML
		 * @param nombreCualif La clave XML
		 * @param atributos Datos asociados a la clave XML
		 */
		public void startElement(String uri, String nombreLocal, String nombreCualif, Attributes atributos) throws SAXException {
			cadena.setLength(0);
		}

		/** 
		 * Metodo que corta y rejunta trozos de claves
		 * @param ch Trozo de cadena a rejuntar
		 * @param comienzo posicion desde donde debe empezar a copiar
		 * @param lon longitud del trozo a copiar
		 */
		public void characters(char ch[], int comienzo, int lon){
			cadena.append(ch,comienzo,lon);
		}
		/** 
		 * Metodo determina que se ha terminado de analizar una clave
		 * @param uri Cadena de la direccion que termina
		 * @param nombreLocal El dato que viene dentro de la clave XML
		 * @param nombreCualif La clave XML
		 */
		public void endElement(String uri, String nombreLocal, String nombreCualif) throws SAXException{
			if(nombreLocal.equals("type") && cadena.toString().equals("administrative_area_level_2")){
				//si es esta area tenemos la provincia
				if(provincia.equals("Alicante")){
					totalResults ="Alicante";
				}
				else{
					totalResults ="Otro";
				}
			}
			else{
				//comprobemos si es un long_name
				if(nombreLocal.equals("long_name")){
					provincia = cadena.toString();

				}
			}
		}


	}


	/** 
	 * Metodo que permite determinar si hay conexion a Internet o no
	 * @param ctx Contexto donde se ejecuta la funcion
	 */
	public static boolean verificaConexion(Context ctx) {
		boolean bConectado = false;
		ConnectivityManager connec = (ConnectivityManager) ctx
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		// No slo wifi, tambin GPRS
		NetworkInfo[] redes = connec.getAllNetworkInfo();
		// este bucle debera no ser tan apa
		for (int i = 0; i < 2; i++) {
			// Tenemos conexin? ponemos a true
			if (redes[i].getState() == NetworkInfo.State.CONNECTED) {
				bConectado = true;
			}
		}
		return bConectado;
	}

	/** 
	 * Metodo que permite empezar o continuar la actividad
	 */
	protected void onResume(){
		super.onResume();
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esmusica = pref.getBoolean("musica", true);
		if(esmusica == true){
			musica = MediaPlayer.create(this,R.raw.relajante);
			musica.start();
		}
	}

	/** 
	 * Metodo que permite pausar la actividad
	 */
	protected void onPause(){
		super.onPause();
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esmusica = pref.getBoolean("musica", true);
		if(esmusica == true){
			musica.stop();
		}
	}

	/** 
	 * Metodo que permite actualizar la localizacion si es mejor a la ya guardada
	 * @param localiz Localizacion GPS obtenida
	 */
	private void actualizaMejorLocaliz(Location localiz){
		if(localiz != null && (mejorLocaliz == null || localiz.getAccuracy() < 2*mejorLocaliz.getAccuracy() || localiz.getTime() - mejorLocaliz.getTime() > DOS_MINUTOS)){
			mejorLocaliz = localiz;
		}
	}

	/** 
	 * Metodo que permite animar el boton de creditos y llamar la actividad de creditos
	 * @param v Vista sobre la que se llam a la funcion
	 */
	public void iraCreditos(View v){
		cargarSonidoBoton();
		Log.e("CREDITOS","CREDITOS");
		animarBoton((Button)findViewById(R.id.button2));
		//hay que programar una tarea y lanzar los creditos
		//ejecutaremos el Intent en un nuevo Hilo para poder ejecutar la animacion
		NuevaTarea td = new NuevaTarea(CreditosActivity.class);
		td.start();		
	}
	/** 
	 * Metodo que permite animar el boton de Clave y llamar la actividad de Clave
	 * @param v Vista sobre la que se llam a la funcion
	 */
	public void iraBuscaClave(View v){
		cargarSonidoBoton();
		animarBoton((Button)findViewById(R.id.button1));
		NuevaTarea td = new NuevaTarea(Clave.class);
		td.start();
	}

	/** 
	 * Metodo que permite animar el boton de Zona y llamar la actividad de Zona
	 * @param v Vista sobre la que se llam a la funcion
	 */
	public void iraBuscaZona(View v){
		cargarSonidoBoton();
		animarBoton((Button)findViewById(R.id.button3));
		NuevaTarea td = new NuevaTarea(Zona.class);
		td.start();
	}

	/** 
	 * Metodo que permite animar el boton de Indice y llamar la actividad de Indice
	 * @param v Vista sobre la que se llam a la funcion
	 */
	public void iraBusquedaIndice(View v){
		cargarSonidoBoton();
		animarBoton((Button)findViewById(R.id.button4));
		NuevaTarea td = new NuevaTarea(Indice.class);
		td.start();
	}

	/** 
	 * Metodo que permite animar el boton de Configurar y llamar la actividad de Configurar
	 * @param v Vista sobre la que se llam a la funcion
	 */
	public void lanzarconfigurar(View v){
		cargarSonidoBoton();
		animarBoton((Button)findViewById(R.id.button5));
		NuevaTarea td = new NuevaTarea(Preferencias.class);
		td.start();
	}

	/** 
	 * Metodo que permite reproducir el sonido del boton al ser pulsado
	 */
	private void cargarSonidoBoton(){
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esSonido = pref.getBoolean("sonido", true);
		if(esSonido){
			sP.play(sonidoboton, 1.0f, 1.0f, 1, 0, 1);
		}
	}

	/** 
	 * Metodo que permite animar el boton que se le pasa por parametro
	 * @param boton Boton que ha sido pulsado
	 */
	private void animarBoton(Button boton){
		Animation animacion = AnimationUtils.loadAnimation(this, R.anim.animacion);
		boton.startAnimation(animacion);
	}

	/**
	 * Clase que permite ejecutar la animacion de los botones al ser pulsados
	 * @version 1.0, 05/11/14
	 * @author Jose Manuel Castellano Dominguez
	 */
	private class NuevaTarea extends Thread{

		private Class class1;

		/** 
		 * Constructor de la nueva tarea a ejecutar
		 * @param class1 clase que debera ejecutarse la nueva tarea
		 */
		public NuevaTarea(Class class1) {
			// TODO Auto-generated constructor stub
			this.class1 = class1;	
		}
		/** 
		 * Metodo que permite la ejecucion de la nueva tarea
		 */
		public void run(){
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			lanzaractividad(class1);
		}

	}

	/** 
	 * Metodo que lanza la actividad que se le pasa por parametro
	 * @param class1 clase que debera ejecutarse la nueva tarea
	 */
	public void lanzaractividad(Class class1){
		Intent e = new Intent(this,class1);
		startActivity(e);
	}
	/** 
	 * Metodo que permite determinar si el servicio que se le suministra esta activo o no
	 * @param serviceClass clase servicio que queremos conocer si esta activo o no
	 */
	private boolean isMyServiceRunning(Class<?> serviceClass) {
		ActivityManager manager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
		for (RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)) {
			if (serviceClass.getName().equals(service.service.getClassName())) {
				return true;
			}
		}
		return false;
	}


	/** 
	 * Metodo que permite crear un menu en la actividad
	 * @param menu Menu que debe inflarse para mostrarse
	 */
	@Override 
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu, menu);
		return true; 

	}


	/** 
	 * Metodo que permite clickar sobre las opciones del men
	 * @param item Opcion del menu pulsada
	 */
	@Override 
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.acercaDe:
			//ir al acercade
			startActivity(new Intent(this,AcercaDe.class));
			break;
		case R.id.config:
			//ir a las preferencias
			startActivity(new Intent(this,Preferencias.class));
			break;
		case R.id.usarLupa:
			//ir a la lupa
			startActivity(new Intent(this,Camara.class));
			break;
		case R.id.mapa:
			//ir al mapa
			startActivity(new Intent(this,MapaPlantas.class));
			break;
		}
		return true; 
	}
}
