package com.plantas.plantasalicante;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnInfoWindowClickListener;
import com.google.android.gms.maps.GoogleMap.OnMapClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentActivity;
import android.util.Log;

/**
 * Clase que muestra el mapa de oogle con las plantas y su info
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class MapaPlantas extends FragmentActivity implements OnMapClickListener, OnInfoWindowClickListener {
	private GoogleMap mapa;
	private Location mejorLocaliz;
	private static final long DOS_MINUTOS = 2*60*1000;
	private Vector<String> almacen;
	private LatLng loca;

	/** 
	 * Crea la actividad de Mapa
	 * @param savedInstanceState Estado anterior de la actividad.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_mapa_plantas);
		mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map)).getMap();
		mapa.setMapType(GoogleMap.MAP_TYPE_SATELLITE);
		//recuperamos la ultima coordenada GPS
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esGPS = pref.getBoolean("permitirgps", true);
		LocationManager manejador = (LocationManager) getSystemService(LOCATION_SERVICE);
		if(esGPS){
			String opcion = pref.getString("tipos", "2");
			if(manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) && (opcion.equals("0") || opcion.equals("2"))){

				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.GPS_PROVIDER));
			}
			if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && (opcion.equals("1") || opcion.equals("2"))){
				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
			}
			if(mejorLocaliz!=null){
				mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(mejorLocaliz.getLatitude(),mejorLocaliz.getLongitude()), 15));
			}
			else{
				//lo centraremos en alcoy
				mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(38.42,0.28), 15));
			}
		}
		else{
			//lo centraremos en alcoy (aunque no sea verdad)
			mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(38.42,0.28), 17));
		}
		mapa.setMyLocationEnabled(true);
		mapa.getUiSettings().setZoomControlsEnabled(true);
		mapa.getUiSettings().setCompassEnabled(true);
		mapa.setOnMapClickListener(this);
		//ahora que nos hemos centrado debemos cargar TODAS LAS PLANTAS
		loadAllPlants();
		mapa.setOnInfoWindowClickListener(this);
		
	}

	/** 
	 * Metodo que permite actualizar la localizacion si es mejor a la ya guardada
	 * @param localiz Localizacion GPS obtenida
	 */
	private void actualizaMejorLocaliz(Location localiz){
		if(localiz != null && (mejorLocaliz == null || localiz.getAccuracy() < 2*mejorLocaliz.getAccuracy() || localiz.getTime() - mejorLocaliz.getTime() > DOS_MINUTOS)){
			mejorLocaliz = localiz;
		}
	}

	/** 
	 * Metodo que permite guardar notas del usuario en la localizacion clickada
	 * @param arg0 Localizacion clickada en el mapa
	 */
	@Override
	//meotod que agregara referencia se plantas
	public void onMapClick(LatLng arg0) {
		// TODO Auto-generated method stub
		//se agregara al fichero
		RutaDialog dialog = new RutaDialog();
		loca = arg0;
		dialog.show(getSupportFragmentManager(), "dialogo");
		
	}
	
	/** 
	 * Metodo que que guarda la nueva nota en el fichero de notas
	 * @param texto Texto de la nota a guardar
	 */
	public void asignarmarca(String texto){
		//primero la agregamos al fichero
		if(texto.equals("cancelar")){
			return;
		}
		String fichero="marcas.txt";
		Double latitude = loca.latitude;
		Double longitude = loca.longitude;
		String textin = "" + latitude + " " + longitude + " " + texto + "\r\n";
		Log.e("Plantas",textin);
		FileOutputStream fos;
		try{
			fos = openFileOutput(fichero,Context.MODE_APPEND);
			fos.write(textin.getBytes());
			fos.close();
			mapa.addMarker(new MarkerOptions().position(loca).icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)).title(texto));
			return;
		}
		catch(FileNotFoundException e){
			Log.e("Plantas Alicante", e.getMessage(),e);
		}
		catch(IOException e){
			Log.e("Plantas Alicante", e.getMessage(),e);
		}
	}
	
	/** 
	 * Metodo que carga todas las plantas y sus localizaciones en el mapa asi como las notas del usuario
	 */
	//metodo que agrega TODOS los lugares registrados
	private void loadAllPlants(){
		//ahora cargaremos las opciones del usuario
		String fichero ="marcas.txt";
		FileInputStream fos;
		try{
			fos = openFileInput(fichero);
			if(fos!=null){
			BufferedReader entrada = new BufferedReader(new InputStreamReader(fos));
			String linea;
			do{
				linea = entrada.readLine();
				if(linea != null){
					//ahora hay que aadir la marca
					int pos = linea.indexOf(" ");
					int pos2 = linea.substring(pos+1).indexOf(" ");
					String latitude = linea.substring(0, pos);
					String longitude = linea.substring(pos+1, pos+1+pos2);
					String textin = linea.substring(pos+2+pos2);
					Double dlatitude = 0.0;
					Double dlongitude = 0.0;
					try{
						dlatitude = Double.parseDouble(latitude);
						dlongitude = Double.parseDouble(longitude);
						LatLng lot = new LatLng(dlatitude, dlongitude);
						mapa.addMarker(new MarkerOptions().position(lot).icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)).title(textin));
					}
					catch(Exception e){
						Log.e("Plantas",e.getMessage(),e);
					}
					
				}
				
			} while(linea != null);
			fos.close();
		}}
		catch(FileNotFoundException e){
			Log.e("Plantas Alicante", e.getMessage(),e);
		}
		catch(IOException e){
			Log.e("Plantas Alicante", e.getMessage(),e);
		}
		//ahora cargaremos la plantas
	    almacen = new AlmacenPlantas(this).listaPlantasZonas();
		for(int i = 0; i < almacen.size(); i++){
			String aux = almacen.get(i);
			int pos = aux.indexOf(";");
			int pos2 = aux.indexOf("****");
			int posfinal = aux.substring(pos+1).indexOf(";");
			String[] unilugares = aux.substring(pos+1,pos+1 +posfinal).split(",,");
			String nombre_comun = aux.substring(0, pos2);
			String nombre_cientifico = aux.substring(pos2+4,pos);
			String tipo = aux.substring(pos + 2 + posfinal);
			Log.e("Plantas","Tipo " + nombre_comun + " " +  tipo );
			for(int indice = 0; indice < unilugares.length; indice++){
				String[] lugarmirar = unilugares[indice].split(",");
				//ahora tenemos los dos datos, los convertiremos a una localizacin
				double latitude = 0;
				double longitude = 0;
				try{
					latitude = Double.parseDouble(lugarmirar[0]);
					longitude = Double.parseDouble(lugarmirar[1]);
				}
				catch(Exception e){
					//ignoramos
					continue;
				}
			Bitmap flor = BitmapFactory.decodeResource(getResources(),R.drawable.florin);
			Bitmap arbol = BitmapFactory.decodeResource(getResources(),R.drawable.arbolin);
			Bitmap hierba = BitmapFactory.decodeResource(getResources(),R.drawable.hierba);
			Bitmap mata = BitmapFactory.decodeResource(getResources(),R.drawable.mata);
			Bitmap arbusto = BitmapFactory.decodeResource(getResources(),R.drawable.arbustin);
			Bitmap plantafinal = null;
			if(tipo.equals("flor")){
				plantafinal=flor;
			}
			else{
				if(tipo.equals("arbol")){
					plantafinal=arbol;
				}
				else{
					if(tipo.equals("mata")){
						plantafinal=mata;
					}
					else{
						if(tipo.equals("arbusto")){
							plantafinal=arbusto;
						}
						else{
							plantafinal=hierba;
						}
					}
				}
			}
			mapa.addMarker(new MarkerOptions().position(new LatLng(latitude,longitude)).icon(BitmapDescriptorFactory.fromBitmap(plantafinal)).title(nombre_comun).snippet(nombre_cientifico));
			}
		}
	}

	/** 
	 * Metodo que permite ir a la actividad de la planta pulsada
	 * @param marker Marcador que fue pulsado
	 */
	@Override
	public void onInfoWindowClick(Marker marker) {
		// TODO Auto-generated method stub
		if(marker.getSnippet()!=null){
		Intent intent = new Intent(this,Ver_Planta.class);
		intent.putExtra("planta", marker.getTitle());
		startActivity(intent);
		}
	}
}
