package com.plantas.plantasalicante;

import java.util.Vector;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * Clase que que permite la representacion de las plantas en Zona e Indice
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class MiAdaptador extends BaseAdapter {
	private final Activity actividad;
	private Vector<String> nombre_comun;
	private Vector<String> nombre_cientifico;
	private Vector<Integer> distancias;
	private Vector<String> tipo;
	private int opcion;
	

	/** 
	 * Constructor del Adaptador
	 * @param actividad Actividad que ha solicitado el adaptador
	 * @param nombre_comun Listado de Nombres comunes de Plantas
	 * @param nombre_cientifico Listado de nombres cientificos de plantas
	 * @param tipo Listado de tipos de plantas
	 * @param opcion Permite identificar el tipo de Lista que se desea
	 */
	public MiAdaptador(Activity actividad, Vector<String> nombre_comun, Vector<String> nombre_cientifico, Vector<String> tipo, int opcion){
		super();
		this.actividad=actividad;
		this.nombre_comun =nombre_comun;
		this.nombre_cientifico = nombre_cientifico;
		this.opcion = opcion;
		this.distancias = null;
		this.tipo = tipo;
	}
	
	/** 
	 * Constructor del Adaptador
	 * @param actividad Actividad que ha solicitado el adaptador
	 * @param nombre_comun Listado de Nombres comunes de Plantas
	 * @param nombre_cientifico Listado de nombres cientificos de plantas
	 * @param tipo Listado de tipos de plantas
	 * @param distancias Listado con las distancias a la que estan las plantas
	 * @param opcion Permite identificar el tipo de Lista que se desea
	 */
	public MiAdaptador(Activity actividad, Vector<String> nombre_comun, Vector<String> nombre_cientifico,Vector<String> tipo, Vector<Integer> distancias, int opcion){
		super();
		this.actividad=actividad;
		this.nombre_comun =nombre_comun;
		this.nombre_cientifico = nombre_cientifico;
		this.opcion = opcion;
		this.distancias = distancias;
		this.tipo=tipo;
	}


	/** 
	 * Retorna cuantas plantas hay en el listado
	 */
	@Override
	public int getCount() {
		// TODO Auto-generated method stub
		return nombre_comun.size();
	}


	/** 
	 * Devuelve el objeto que hay en la posicion que se le pasa por parametro
	 * @param arg0 Posicion de la lista que se desee recuperar.
	 */
	@Override
	public Object getItem(int arg0) {
		// TODO Auto-generated method stub
		return nombre_comun.elementAt(arg0);
	}

	/** 
	 * Devuelve el id que hay en la posicion que se le pasa por parametro
	 * @param position Posicion de la lista que se desee recuperar.
	 */
	@Override
	public long getItemId(int position) {
		// TODO Auto-generated method stub
		return position;
	}

	/** 
	 * Devuelve la vista asociada a la posicion que se le pasa por parametro
	 * @param position  Posicion de la lista que se desee recuperar.
	 * @param convertView Vista convertida
	 * @param parent Acceso al conjunto de vistas padres de la actual
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		// TODO Auto-generated method stub
		LayoutInflater inflater = actividad.getLayoutInflater();
		View view = null;
		if(opcion == 0){
		view = inflater.inflate(R.layout.elemento_lista, null,true);
		}
		else{
			view = inflater.inflate(R.layout.elemento_lista_zona, null,true);
		}
		TextView textView = (TextView)view.findViewById(R.id.titulo);
		textView.setText(nombre_comun.elementAt(position));
		TextView textView2 = (TextView)view.findViewById(R.id.subtitulo);
		textView2.setText(nombre_cientifico.elementAt(position));
		ImageView imageView = (ImageView)view.findViewById(R.id.icono);
		//segun lo que ponga en el campo debemos poner un icono u otro
		if(tipo.elementAt(position).equals("flor")){
		imageView.setImageResource(R.drawable.florin);
		}
		else{
			if(tipo.elementAt(position).equals("arbusto")){
			imageView.setImageResource(R.drawable.arbustin);
			}
			else{
				if(tipo.elementAt(position).equals("hierba")){
					imageView.setImageResource(R.drawable.hierba);
					}
				else{
					if(tipo.elementAt(position).equals("arbol")){
						imageView.setImageResource(R.drawable.arbolin);
						}
					else{
						imageView.setImageResource(R.drawable.mata);
					}
				}
			}
		}
		if(opcion == 1){
			//nos queda por poner la distancia
			TextView textView3 = (TextView)view.findViewById(R.id.distancia);
			if(distancias.elementAt(position) < 1000){
				textView3.setText(distancias.elementAt(position) + " M");
			}
			else{
			textView3.setText(distancias.elementAt(position)/1000 + " Km");
			}
		}
		
		//aqui debemos sustituirlo con la imagen web
		return view;
	}
	
	/** 
	 * Establece el lista de nombres comunes
	 * @param nombre_comun listado de nombres comunes
	 */
	public void setNombreComun(Vector<String> nombre_comun){
		this.nombre_comun= nombre_comun;
	}
	
	/** 
	 * Establece el lista de nombres cientificos
	 * @param nombre_cientifico listado de nombres cientificos
	 */
	public void setNombreCientifico(Vector<String> nombre_cientifico){
		this.nombre_cientifico= nombre_cientifico;
	}
	
	/** 
	 * Establece la lista de distancias
	 * @param distancias listado de distancias
	 */
	public void setDistancias(Vector<Integer> distancias){
		this.distancias= distancias;
	}
	
	/** 
	 * Establece la lista de tipos
	 * @param tipo listado de tipos
	 */
	public void setTipo(Vector<String> tipo){
		this.tipo = tipo;
	}

}
