package com.plantas.plantasalicante;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * Clase que permite crear y gestionar la base de datos para las claves
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class PlantasClaveSQL extends SQLiteOpenHelper {
	private final Context context;
	String sqlCreate = "CREATE TABLE ClavePlantas " +
            "(_id INTEGER PRIMARY KEY, " +
            " texto TEXT)";
	
	String sqlCreate2 = "CREATE TABLE ClavePlantasOpcionRespuesta " +
            "(_id INTEGER PRIMARY KEY, " +
            " primera INTEGER, " +
            " segunda INTEGER, " +
            " tercera INTEGER, " +
            " cuarta INTEGER, " +
            " quinta INTEGER)";
	
	String sqlCreate3 = "CREATE TABLE ClavePlantasOpcionPlanta " +
            "(_id INTEGER PRIMARY KEY, " +
            " planta TEXT)";

	/** 
	 * Constructor de la base de datos
	 * @param context Contexto de la actividad en la que se solicita la base de datos
	 */
	public PlantasClaveSQL(Context context){
		super(context, "plantasclave",null,1);
		this.context=context;
	}

	/** 
	 * Construye la base de datos en caso de que no este creada en el dispositivo.
	 * @param db Base de datos que se puede leer o escribir
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		// TODO Auto-generated method stub
		//hay que crear las bases de datos
		db.execSQL(sqlCreate);
		db.execSQL(sqlCreate2);
		db.execSQL(sqlCreate3);
		//hay que insertar los datos, se introducir una linea por planta
		//tenemos que acceder al fichero para construir la base de datos
		leerDatos(db,R.raw.datosclave);
		leerDatos(db,R.raw.datosopcionrespuesta);
		leerDatos(db,R.raw.datosopcionplanta);
	   
		
	}

	/** 
	 * Actualiza la base de datos en caso de que este obsoleta
	 * @param db Base de datos que se puede leer o escribir
	 * @param oldVersion version de la antigua base de datos
	 * @param newVersion version de la base de datos ms nueva
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// TODO Auto-generated method stub
		db.execSQL("DROP TABLE IF EXISTS ClavePlantas");
		db.execSQL("DROP TABLE IF EXISTS ClavePlantasOpcionRespuesta");
		db.execSQL("DROP TABLE IF EXISTS ClavePlantasOpcionPlanta"); 
        //Se crea la nueva versin de la tabla
		db.execSQL(sqlCreate);
		db.execSQL(sqlCreate2);
		db.execSQL(sqlCreate3);
		leerDatos(db,R.raw.datosclave);
        leerDatos(db,R.raw.datosopcionrespuesta);
        leerDatos(db,R.raw.datosopcionplanta);
	}
	
	/** 
	 * lee los datos de comandos SQL para crear los datos
	 * @param db Base de datos que se puede escribir
	 * @param fichero Fichero en memoria que contiene las sentencias SQL a leer
	 */
	private void leerDatos(SQLiteDatabase db, int fichero){
		 try {
		    	InputStream f = context.getResources().openRawResource(fichero);
		    	
	            BufferedReader entrada = new BufferedReader(new InputStreamReader(f));
	            String linea;
	            do {
	                   linea = entrada.readLine();
	                   if (linea != null) {
	                         db.execSQL(linea);
	                   }
	            } while (linea != null);
	            f.close();
	     } catch (Exception e) {
	            Log.e("Plantas", e.getMessage(), e);

	     }
			
	}
	
}
