package com.plantas.plantasalicante;

import android.content.ContentProvider;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;


/**
 * Clase que gestiona el Content Provider de la aplicacion
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class PlantasProvider extends ContentProvider {
	public static final String AUTORIDAD = "com.plantas.plantasalicante";
	public static final Uri CONTENT_URI = Uri.parse("content://" + AUTORIDAD + "/plantas");
	public static final int TODOS_LOS_ELEMENTOS = 1;
	public static final int UN_ELEMENTO = 2;
	private static UriMatcher URI_MATCHER = null;
	static {
		URI_MATCHER = new UriMatcher(UriMatcher.NO_MATCH);
		URI_MATCHER.addURI(AUTORIDAD, "plantas", TODOS_LOS_ELEMENTOS);
		URI_MATCHER.addURI(AUTORIDAD, "plantas/#", UN_ELEMENTO);
		URI_MATCHER.addURI(AUTORIDAD, "plantas/*", UN_ELEMENTO);
	}
	
	public static final String TABLA = "Plantas";
	private SQLiteDatabase baseDeDatos;
	
	/** 
	 * Constructor del Content Provider
	 */
	public boolean onCreate(){
		PlantasSQLiteHelper dbHelper = new PlantasSQLiteHelper(getContext());
		baseDeDatos = dbHelper.getWritableDatabase();
		return baseDeDatos != null && baseDeDatos.isOpen();
	}

	
	/** 
	 * Metodo para borrar informacion del Content Provider (NO SE USA)
	 * @param uri URI proporcionada con la operacion a realizar
	 * @param seleccion Campos que son seleccionados para el borrado
	 * @param argSeleccion valores de los campos seleccionados para el borrado
	 */
	@Override
	public int delete(Uri uri, String seleccion, String[] argSeleccion) {
		//no se van a borrar datos, no se utiliza
		return 0;
	}

	/** 
	 * Metodo para obtener el tipo
	 * @param uri URI proporcionada con la operacion a realizar
	 */
	@Override
	public String getType(final Uri uri) {
		// TODO Auto-generated method stub
		switch(URI_MATCHER.match(uri)){
		case TODOS_LOS_ELEMENTOS:
			return "vnd.android.cursor.dir/vnd.com.plantas.planta";
		case UN_ELEMENTO:
			return "vnd.android.cursor.item/vnd.com.plantas.planta";
		default:
			throw new IllegalArgumentException("URI incorrecta: " + uri);
		}
	}

	/** 
	 * Metodo para insertar un dato en el content Provider (No se USA)
	 * @param uri URI proporcionada con la operacion a realizar
	 * @param valores valores a insertar
	 */
	@Override
	public Uri insert(Uri uri, ContentValues valores) {
		return null;
	}

	/** 
	 * Metodo para consultar informacion del ContentProvider
	 * @param uri URI proporcionada con la operacion a realizar
	 * @param proyeccion Proyeccion realizada sobre la tabla para hacer la consulta
	 * @param seleccion Campos que son seleccionados para la consulta
	 * @param argSeleccion valores de los campos seleccionados para la consulta
	 */
	@Override
	public Cursor query(Uri uri, String[] proyeccion, String seleccion, String[] argSeleccion,
			String orden) {
		SQLiteQueryBuilder queryBuilder = new SQLiteQueryBuilder();
		queryBuilder.setTables(TABLA);
		switch(URI_MATCHER.match(uri)){
		case TODOS_LOS_ELEMENTOS:
			break;
		case UN_ELEMENTO:
			String nombre = uri.getPathSegments().get(1);
			queryBuilder.appendWhere("nombre_comun = '" + nombre + "'");
			break;
		default:
			throw new IllegalArgumentException("URI incorrecta " + uri);
		}
		return queryBuilder.query(baseDeDatos, proyeccion, seleccion, argSeleccion, null, null, orden);
	}

	/** 
	 * Metodo para actualizar informacion del Content Provider (NO SE USA)
	 * @param uri URI proporcionada con la operacion a realizar
	 * @param valores Valores a actualizar
	 * @param seleccion Campos que son seleccionados para la actualizacion
	 * @param argSeleccion valores de los campos seleccionados la actualizacion
	 */
	@Override
	public int update(Uri uri, ContentValues valores, String seleccion, String[] ArgumentosSeleccion) {
		return 0;
	}
}
