package com.plantas.plantasalicante;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * Clase que permite crear y gestionar la base de datos de las plantas que va asociada al Content Provider
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class PlantasSQLiteHelper extends SQLiteOpenHelper {
	private final Context context;
	String sqlCreate = "CREATE TABLE Plantas " +
            "(_id INTEGER PRIMARY KEY AUTOINCREMENT, " +
            " nombre_comun TEXT, " +
            " nombre_cientifico TEXT, " +
            " lugares TEXT, " +
            " familia TEXT, " + 
            " descripcion TEXT, " +
            " floracion TEXT, " +
            " distribucion TEXT, " +
            " ecologia TEXT, " +
            " altura TEXT, " +
            " abundancia TEXT, " +
            " uicn TEXT, " +
            " biotipo TEXT, " +
            " tipo TEXT  )";

	/** 
	 * Constructor de la base de datos
	 * @param context Contexto de la actividad en la que se solicita la base de datos
	 */
	public PlantasSQLiteHelper(Context context){
		super(context, "plantas",null,1);
		this.context=context;
	}

	/** 
	 * Construye la base de datos en caso de que no este creada en el dispositivo.
	 * @param db Base de datos que se puede leer o escribir
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		// TODO Auto-generated method stub
		//hay que crear las bases de datos
		db.execSQL(sqlCreate);
		//hay que insertar los datos, se introducir una linea por planta
		//tenemos que acceder al fichero para construir la base de datos
		leerDatos(db);
	   
		
	}

	/** 
	 * Actualiza la base de datos en caso de que este obsoleta
	 * @param db Base de datos que se puede leer o escribir
	 * @param oldVersion version de la antigua base de datos
	 * @param newVersion version de la base de datos ms nueva
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// TODO Auto-generated method stub
		db.execSQL("DROP TABLE IF EXISTS Plantas");
		db.execSQL(sqlCreate);
        //Se crea la nueva versin de la tabla
        leerDatos(db);
	}
	
	/** 
	 * lee los datos de comandos SQL para crear los datos
	 * @param db Base de datos que se puede escribir
	 */
	private void leerDatos(SQLiteDatabase db){
		 try {
		    	InputStream f = context.getResources().openRawResource(R.raw.datos);
	            BufferedReader entrada = new BufferedReader(new InputStreamReader(f));
	            String linea;
	            do {
	                   linea = entrada.readLine();
	                   if (linea != null) {
	                         db.execSQL(linea);
	                   }
	            } while (linea != null);
	            f.close();
	     } catch (Exception e) {
	            Log.e("Plantas", e.getMessage(), e);

	     }
			
	}
	
}
