package com.plantas.plantasalicante;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.Calendar;
import java.util.Timer;
import java.util.TimerTask;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.net.Uri;
import android.os.Environment;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;

/**
 * Clase que permite representar el servicio de Actualizacion
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class ServicioActualizacion extends Service {
	private NotificationManager nm = null;
	private static final int ID_NOTIFICACION_CREAR = 1;
	private Context context;
	private Calendar calaux;
	private String FICHERO = Environment.getExternalStorageDirectory() + "/Android/data/com.plantas.plantasalicante/files/tiemposervicio.txt";

	/** 
	 * Constructor del Servicio
	 */
	public ServicioActualizacion() {
		super();
		context = this;
	}
	/** 
	 * Ejecucion del Servicio
	 * @param intent Intencion asociada al servicio
	 * @param flags Parametros que vienen asociados al servicio
	 * @param startId Id con el que empieza el servicio
	 */
	@Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId); 
        onHandleIntent(intent);
        return START_STICKY;
    }
	
	/** 
	 * Metodo que programa las diversas ejecucion del servicio
	 * @param intent Intencion asociada al servicio
	 */
	protected void onHandleIntent(Intent intent) {
		// TODO Auto-generated method stub
		//rescatamos las preferencias
		int min = 0;
		int delayed = 0;
		//comprobamos si hay acceso
		String stadoSD = Environment.getExternalStorageState();
		if(stadoSD.equals(Environment.MEDIA_MOUNTED)){
		File ruta = new File(Environment.getExternalStorageDirectory() + "/Android/data/com.plantas.plantasalicante/files/");
		if (!ruta.exists()) {
			ruta.mkdirs();
		}

		calaux = Calendar.getInstance();
		SharedPreferences pref= PreferenceManager.getDefaultSharedPreferences(this);
		String minutos = pref.getString("ubicar", "1440");
		min = 0;
		try{
			min = Integer.parseInt(minutos);
		}
		catch(Exception e){
			min = 0;
		}
		if(min == 0){
			//no debe actualizarse
			return;
		}
		
		//ver a que hora es la siguiente ejecucion
		delayed = 0;
		Calendar cal1 = Calendar.getInstance();
		long mili1 = cal1.getTimeInMillis();
		Calendar cal2 = Calendar.getInstance();
		long mili2;
		FileInputStream fos;
		try{
			fos = new FileInputStream(FICHERO);
			BufferedReader entrada = new BufferedReader(new InputStreamReader(fos));
			String linea;
			linea = entrada.readLine();
			//la entrada esta en formato YEAR:MON:DAY:HOUR:MINUTE:SECOND
			String[] fecha =linea.split(":");
			try{
				cal2.set(Integer.parseInt(fecha[0]), Integer.parseInt(fecha[1])-1, Integer.parseInt(fecha[2]), Integer.parseInt(fecha[3]), Integer.parseInt(fecha[4]), Integer.parseInt(fecha[5]));
				mili2 = cal2.getTimeInMillis();
			}
			catch(Exception e){
				mili2 = mili1;
			}
			entrada.close();
		}
		catch(Exception e){
			mili2 = mili1;
		}
		if(mili2!=mili1 && ((mili2 + (min * 60 * 1000)) > mili1)){
			delayed = (int) ((mili2 + (min * 60 * 1000)) - mili1);
			
		}
		else{
			delayed = 0;
		}}
		else{
			delayed = 0;
		}

		Timer timer = new Timer();
		timer.scheduleAtFixedRate(new TimerTask() {     
			@Override
			public void run() {
				// do something
				//metemos en el calendar la fecha actual
				calaux = Calendar.getInstance();
				FileOutputStream fos;
				String year = "" + calaux.get(Calendar.YEAR);
				String month = "" + (calaux.get(Calendar.MONTH) + 1) ;
				String day = "" + calaux.get(Calendar.DAY_OF_MONTH);
				String hour = "" + calaux.get(Calendar.HOUR_OF_DAY);
				String minute = "" + calaux.get(Calendar.MINUTE);
				String second = "" + calaux.get(Calendar.SECOND);
				String cadena = year + ":" + month + ":" + day + ":" + hour + ":" + minute + ":" + second; 
				try{
					//primero deberiamos borrar el fichero si existe
					fos = new FileOutputStream(FICHERO,false);
					fos.write(cadena.getBytes());
					fos.close();
				}
				catch(Exception e){
					
				}
				//esta tarea debe conectarse a Internet, recuperar el fichero
				new Thread(new Runnable() {
				    public void run() {
				try {
					URL url= new URL("http://www.mi1robot.es/android/version");
					URLConnection yc = url.openConnection();
					BufferedReader in = new BufferedReader(new InputStreamReader(yc.getInputStream()));
					String inputLine = in.readLine();
					PackageInfo pInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
					String version = pInfo.versionName;
					double versionweb = 0.0;
					double versionprograma = 0.0;
					try{
						versionweb = Double.parseDouble(inputLine);
						versionprograma = Double.parseDouble(version);
					}
					catch(Exception e){
						//no es la misma version
						return;
					}
					nm =(NotificationManager)getApplicationContext().getSystemService(Context.NOTIFICATION_SERVICE);
					if(versionweb > versionprograma){
						//hay que crear la notificacion avisando de que hay una actualizacion disponible
						NotificationCompat.Builder notific = new NotificationCompat.Builder(context).setContentTitle("Actualizacin Disponible")
					             .setSmallIcon(R.drawable.ic_launcher)
					             .setContentText("Hay una nueva actualizacion de la aplicacion");

							
							PendingIntent intencionPendiente = PendingIntent.getActivity(context, 0, new Intent(Intent.ACTION_VIEW,Uri.parse("http://www.mi1robot.es")), 0);
							notific.setContentIntent(intencionPendiente);
							nm.notify(ID_NOTIFICACION_CREAR, notific.build());

					}
					else{
						nm.cancel(ID_NOTIFICACION_CREAR);
					}
					return;
					
				} catch(Exception e){
				}
			}}).start();
			}}, delayed, min*60*1000);


	}

	/** 
	 * Metodo que nos permite bindear el servicio (No se bindea)
	 * @param intent Intencion asociada al servicio
	 */
	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

}
