package com.plantas.plantasalicante;

import java.io.IOException;

import android.content.Context;
import android.hardware.Camera;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
/**
 * Clase que permite montar una superficie donde mostrar los datos de la camara
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class ShowCamera extends SurfaceView implements SurfaceHolder.Callback {

   private SurfaceHolder holdMe;
   private Camera theCamera;

	/** 
	 * Constructor de la camara
	 * @param context Contexto donde se ejecuta esta clase
	 * @param camera Objeto camara que ser utilizado
	 */
   public ShowCamera(Context context,Camera camera) {
      super(context);
      theCamera = camera;
      holdMe = getHolder();
      holdMe.addCallback(this);
      //info en Internet, esto es necesario para versiones inferiores a 3.0
      holdMe.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
      
   }

	/** 
	 * Metodo para indicar que la superficie ha cambiado
	 * @param arg0 Nueva superficie
	 * @param arg1 Ancho de la nueva superficie
	 * @param arg2 Largo d ela nueva superficie
	 * @param arg3 Alto de la nueva superficie
	 */
   @Override
   public void surfaceChanged(SurfaceHolder arg0, int arg1, int arg2, int arg3) {
	   if (holdMe.getSurface() == null){ // preview surface does not exist 
		   return; 
		   }
	   try { 
		   theCamera.stopPreview(); 
		   } 
	   catch (Exception e){ 
		   // ignoramos ya que puede ser que estuviera detenida ya
		   }
	   
        //tecnicamente se ha hecho zoom
	   try { 
		   theCamera.setPreviewDisplay(holdMe); 
		   theCamera.startPreview(); 
		   } 
	   catch (Exception e){ 
		   e.printStackTrace(); 
	   }
		  
		   
   }

   /** 
	 * Metodo para indicar que una nueva superficie se ha creado
	 * @param holder Nueva superficie
	 */
   @Override
   public void surfaceCreated(SurfaceHolder holder) {
      try   {
         theCamera.setPreviewDisplay(holder);
         theCamera.startPreview(); 
      } catch (IOException e) {
    	  e.printStackTrace();
      }
   }

   /** 
  	 * Metodo para indicar que se ha destruido una superficie
  	 * @param arg0 Superficie que ha sido destruida
  	 */
   @Override
   public void surfaceDestroyed(SurfaceHolder arg0) {
   }

}