package com.plantas.plantasalicante;

import java.net.URL;
import java.util.Vector;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Clase que permite representar una lista de plantas de la zona actual
 * @version 1.0, 05/11/14
 * @author Jose Manuel Castellano Dominguez
 */
public class Zona extends ListActivity implements LocationListener {

	private LocationManager manejador;
	private Location mejorLocaliz;
	private static int DOS_MINUTOS = 2*60*1000;
	private Context context;
	private SharedPreferences pref;
	private MiAdaptador miadaptador = new MiAdaptador(this,new Vector<String>(),new Vector<String>(),new Vector<String>(),new Vector<Integer>(),1);

	/** 
	 * Crea la actividad de Zona
	 * @param savedInstanceState Estado anterior de la actividad.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		context = this;
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_zona);

		pref= PreferenceManager.getDefaultSharedPreferences(this);
		Boolean esGPS = pref.getBoolean("permitirgps", true);
		manejador = (LocationManager) getSystemService(LOCATION_SERVICE);
		String opcion = pref.getString("tipos", "2");
		if(esGPS && (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) || manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) )){
			if(manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) && (opcion.equals("0") || opcion.equals("2"))){
				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.GPS_PROVIDER));
			}
			if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && (opcion.equals("1") || opcion.equals("2"))){
				actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
			}
			//vamos a llamar a tareagps
			setListAdapter(miadaptador);
			//nuevaZona();

		}
		else{
			//Dar alarma
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(R.string.missinggpstext)
			.setTitle(R.string.missinggpstitle);
			builder.setPositiveButton(R.string.preferencias, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					// User clicked OK button
					//hay que abrir el men preferencias
					Intent intent = new Intent(context,Preferencias.class);
					startActivity(intent);
				}
			});

			builder.setNeutralButton(R.string.preferenciassistema, new DialogInterface.OnClickListener() {

				@Override
				public void onClick(DialogInterface dialog, int which) {
					// TODO Auto-generated method stub
					startActivity(new Intent(android.provider.Settings.ACTION_LOCATION_SOURCE_SETTINGS));
				}
			});
			builder.setNegativeButton(R.string.cancelar, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					// User cancelled the dialog
					dialog.dismiss();
				}
			});

			AlertDialog alerta = builder.create();
			alerta.show();


		}
	}

	/**
	 * Clase que se ejecutar en otro hilo y permitir recuperar la ubicacion actual
	 * @version 1.0, 05/11/14
	 * @author Jose Manuel Castellano Dominguez
	 */
	private class TareaGPS extends AsyncTask<Integer,Void,String>{
		/** 
		 * Metodo que ejecuta la tarea
		 * @param cantidad Estado anterior de la actividad.
		 */
		protected String doInBackground(Integer... cantidad){
			String result= null;
			try {
				Boolean hayconexion=verificaConexion(context);
				if(!hayconexion){
					//no podemos conectarnos, abortamos
					return null;
				}
				URL url= new URL("https://maps.googleapis.com/maps/api/geocode/xml?latlng=" + mejorLocaliz.getLatitude() + "," + mejorLocaliz.getLongitude() +"&sensor=true");	
				SAXParserFactory fabrica = SAXParserFactory.newInstance();
				SAXParser parser = fabrica.newSAXParser();
				XMLReader lector = parser.getXMLReader();
				ManejadorXML manejadorXML = new ManejadorXML();
				lector.setContentHandler(manejadorXML);
				lector.parse(new InputSource(url.openStream()));
				return manejadorXML.getTotalResults();
			}
			catch(Exception e){
				Log.e("Plantas", "Excepcion:" + e.getMessage(), e);
			}
			return result;
		}
	}

	/**
	 * Clase que permite manejar los datos XML del servicio de geolocalizacion de google
	 * @version 1.0, 05/11/14
	 * @author Jose Manuel Castellano Dominguez
	 */
	public class ManejadorXML extends DefaultHandler{
		private String totalResults;
		private StringBuilder cadena = new StringBuilder();
		private String provincia;
		/** 
		 * Devuelve la cadena encontrada
		 */
		public String getTotalResults(){
			return totalResults;
		}
		/** 
		 * Metodo que empieza a parsear un nuevo elemento
		 * @param uri Cadena de la direccion que empieza
		 * @param nombreLocal El dato que viene dentro de la clave XML
		 * @param nombreCualif La clave XML
		 * @param atributos Datos asociados a la clave XML
		 */
		public void startElement(String uri, String nombreLocal, String nombreCualif, Attributes atributos) throws SAXException {
			cadena.setLength(0);
		}
		/** 
		 * Metodo que corta y rejunta trozos de claves
		 * @param ch Trozo de cadena a rejuntar
		 * @param comienzo posicion desde donde debe empezar a copiar
		 * @param lon longitud del trozo a copiar
		 */
		public void characters(char ch[], int comienzo, int lon){
			cadena.append(ch,comienzo,lon);
		}
		/** 
		 * Metodo determina que se ha terminado de analizar una clave
		 * @param uri Cadena de la direccion que termina
		 * @param nombreLocal El dato que viene dentro de la clave XML
		 * @param nombreCualif La clave XML
		 */
		public void endElement(String uri, String nombreLocal, String nombreCualif) throws SAXException{
			if(nombreLocal.equals("type") && cadena.toString().equals("administrative_area_level_4")){
				//si es esta area tenemos la provincia
				totalResults= provincia;
			}
			else{
				//comprobemos si es un long_name
				if(nombreLocal.equals("long_name")){
					provincia = cadena.toString();

				}
			}
		}


	}
	/** 
	 * Permite solicitar la nueva localizacion obtenida a los proveedores y comparar si es buena respecto a la de antes
	 */
	private void activarProveedores(){
		Boolean esGPS = pref.getBoolean("permitirgps", true);
		String opcion = pref.getString("tipos", "2");
		if(esGPS && (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) || manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) )){
			if(manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) && (opcion.equals("0") || opcion.equals("2"))){
				manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER, 20*1000, 5, this);
			}
			if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && (opcion.equals("1") || opcion.equals("2"))){
				manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 10*1000, 10, this);
			}
		}
	}
	/** 
	 * Metodo que permite empezar o continuar la actividad
	 */
	protected void onResume(){
		super.onResume();
		activarProveedores();
	}
	/** 
	 * Metodo que permite pausar la actividad
	 */
	protected void onPause(){
		super.onPause();
		manejador.removeUpdates(this);
	}

	@Override
	/** 
	 * Metodo que se ejecuta al detectar que el GPS ha cambiado de Zona
	 * @param location Localizacion nueva obtenida
	 */
	public void onLocationChanged(Location location) {
		// TODO Auto-generated method stub
		if(actualizaMejorLocaliz(location)){
			nuevaZona();
		}
	}

	/** 
	 * Metodo que indica el cambio de alguno de los proveedores
	 * @param provider proveedor que ha cambiado de estado
	 * @param status Estatus del proveedor
	 * @param extras informacion extra del cambio
	 */
	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub
		activarProveedores();
	}

	/** 
	 * Metodo que indica que un nuevo proovedor esta disponible
	 * @param provider proveedor que esta disponible
	 */
	@Override
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub
		activarProveedores();
	}

	/** 
	 * Metodo que indica que un proovedor no esta disponible
	 * @param provider proveedor que no esta disponible
	 */
	@Override
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub
		activarProveedores();
	}

	/** 
	 * Metodo que permite actualizar la localizacion si es mejor a la ya guardada
	 * @param localiz Localizacion GPS obtenida
	 */
	private boolean actualizaMejorLocaliz(Location localiz){
		if(localiz != null && (mejorLocaliz == null || localiz.getAccuracy() < 2*mejorLocaliz.getAccuracy() || localiz.getTime() - mejorLocaliz.getTime() > DOS_MINUTOS)){
			mejorLocaliz = localiz;
			return true;
		}
		return false;
	}

	/** 
	 * Metodo que permite determinar si hay conexion a Internet o no
	 * @param ctx Contexto donde se ejecuta la funcion
	 */
	public static boolean verificaConexion(Context ctx) {
		boolean bConectado = false;
		ConnectivityManager connec = (ConnectivityManager) ctx
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		// No slo wifi, tambin GPRS
		NetworkInfo[] redes = connec.getAllNetworkInfo();
		// este bucle debera no ser tan apa
		for (int i = 0; i < 2; i++) {
			// Tenemos conexin? ponemos a true
			if (redes[i].getState() == NetworkInfo.State.CONNECTED) {
				bConectado = true;
			}
		}
		return bConectado;
	}
	/** 
	 * Metodo que permite ejecutar una accion al pulsar uno de los elementos de la lista
	 * @param listView lista sobre la que se ha hecho click
	 * @param vista Vista sobre la que se ha hecho click
	 * @param posicion Numero de elemento sobre el que se ha hecho click
	 * @param id Id del elemento pulsado
	 */
	protected void onListItemClick(ListView listView, View vista, int posicion, long id){
		super.onListItemClick(listView, vista, posicion, id);
		Object o = getListAdapter().getItem(posicion);
		Intent intent = new Intent(this,Ver_Planta.class);
		intent.putExtra("planta", o.toString());
		startActivity(intent);

	}

	/** 
	 * Metodo que permite pasar a la actividad de Mapa
	 * @param v Vista sobre la que se ha pulsado
	 */
	public void vermapa(View v){
		startActivity(new Intent(this,MapaPlantas.class));
	}

	/** 
	 * Metodo que permite cargar la lista de una nueva zona
	 */
	//este metodo se conecta al servicio web y actualiza la pantalla
	private void nuevaZona(){
		TareaGPS tg = new TareaGPS();
		tg.execute(1);
		String lugar = null;
		try {
			lugar = tg.get(4,TimeUnit.SECONDS);
		} catch (InterruptedException e) {
			Log.e("Plantas","Interrumpido el hilo para obtener el sitio");
			lugar = "No encontrado";
		} catch (ExecutionException e) {
			// TODO Auto-generated catch block
			Log.e("Plantas","Excepcion encontrada al obtener el sitio");
			lugar = "No encontrado";
		} catch (TimeoutException e) {
			// TODO Auto-generated catch block
			Log.e("Plantas","Fin de Tiempo");
			lugar = "No encontrado";
		}
		if(lugar == null){
			lugar = "No encontrado";
		}
		TextView nt = (TextView) this.findViewById(R.id.lazona);
		nt.setText(lugar);

		//ahora hay que buscar las plantas por zonas

		//ahora deberiamos buscar las plantas que esten dentro del margen de preferencias
		//sacaremos el info de todas las plantas y hallaremos su distancia respecto al actual
		Vector<String> almacen = new AlmacenPlantas(this).listaPlantasZonas();
		//ahora dividimos
		Vector<String> nombre_comun = new Vector<String>();
		Vector<String> nombre_cientifico = new Vector<String>();
		Vector<Integer> lugares = new Vector<Integer>();
		Vector<String> tipo = new Vector<String>();
		String kilometros = pref.getString("kilometros", "10");
		int distancia = -1;
		try{
			distancia = Integer.parseInt(kilometros);
		}
		catch(Exception e){
			distancia = 10;
		}
		for(int i = 0; i < almacen.size(); i++){
			String aux = almacen.get(i);
			int pos = aux.indexOf(";");
			int pos2 = aux.indexOf("****");
			int posfinal = aux.substring(pos+1).indexOf(";");
			//antes de introducir los datos tenemos que ver si entra dentro de la distancia de preferencias
			//para ello debemos calcular la distancia entra la posicion actual y la del punto del mapa, si esta se incluye la flor
			int distanciacalculada = 9000000;

			String[] unilugares = aux.substring(pos+1,pos+1 +posfinal).split(",,");
			for(int indice = 0; indice < unilugares.length; indice++){
				String[] lugarmirar = unilugares[indice].split(",");
				//ahora tenemos los dos datos, los convertiremos a una localizacin
				Location newLoc = new Location(LocationManager.GPS_PROVIDER);
				double latitude = 0;
				double longitude = 0;
				try{
					latitude = Double.parseDouble(lugarmirar[0]);
					longitude = Double.parseDouble(lugarmirar[1]);
				}
				catch(Exception e){
					//ignoramos
					continue;
				}
				newLoc.setLatitude(latitude);
				newLoc.setLongitude(longitude);
				int distanciares = (int)newLoc.distanceTo(mejorLocaliz);
				if(distanciares < distanciacalculada){
					distanciacalculada = distanciares;
					continue;
				}

			}
			//ahora aqui comprobamos la distancia
			Log.e("Distancia plantas",""+distanciacalculada);
			if(distanciacalculada <= distancia*1000){
				//se agrega
				nombre_comun.add(aux.substring(0, pos2));
				nombre_cientifico.add(aux.substring(pos2+4,pos));
				tipo.add(aux.substring(pos + 2 + posfinal));
				lugares.add(distanciacalculada);
				continue;
			}

		}
		miadaptador.setNombreComun(nombre_comun);
		miadaptador.setNombreCientifico(nombre_cientifico);
		miadaptador.setDistancias(lugares);
		miadaptador.setTipo(tipo);
		miadaptador.notifyDataSetChanged();

	}

	/** 
	 * Metodo que permite crear un menu en la actividad
	 * @param menu Menu que debe inflarse para mostrarse
	 */
	@Override 
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu, menu);
		return true; 

	}


	/** 
	 * Metodo que permite clickar sobre las opciones del men
	 * @param item Opcion del menu pulsada
	 */
	@Override 
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.acercaDe:
			//ir al acercade
			startActivity(new Intent(this,AcercaDe.class));
			break;
		case R.id.config:
			//ir a las preferencias
			startActivity(new Intent(this,Preferencias.class));
			break;
		case R.id.usarLupa:
			//ir a la lupa
			startActivity(new Intent(this,Camara.class));
			break;
		case R.id.mapa:
			//ir al mapa
			startActivity(new Intent(this,MapaPlantas.class));
			break;
		}
		return true; 
	}
}
